#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "basemanager4xi.py">
#<abstract4xi abstract = "provides BaseManagerXi class"/>
#<refer4xi reference = "managers.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
#<note4xi note = "this class does NOT define its own methods because the number of parameters vary 
#                 and calling BaseManager.method or return BaseManager.method might fail"/>
# -------------------------------------------------------------------------------------------------
#

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'BaseManager4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass    
try:                 # most-common ones
    import weakref
    import array
    import queue
    from traceback import format_exc         
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                           
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.managers import BaseManager
except:
    print("basemanager4xi.py *** failed in from multiprocessing.managers import BaseManager ***")
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from authenticationstring4xi import AuthenticationString4Xi
from autoproxy4xi import AutoProxy4Xi
from baseproxy4xi import BaseProxy4Xi
from constant_family4xi import ConstantFamily4Xi
from constant_family4xi import get_default_family4xi
from constant_managers4xi import get_debug_info_manager4xi
from constant_managers4xi import get_dummy_manager4xi
from constant_managers4xi import get_initial_manager4xi
from constant_managers4xi import get_started_manager4xi
from constant_managers4xi import get_number_of_objects_manager4xi
from constant_managers4xi import get_shutdown_manager4xi
from current_process4xi import get_authkey_current_process4xi
from dispatch4xi import dispatch4xi
from finalize4xi import Finalize4Xi
from listener_client4xi import listener_client4xi
from pipe4xi import Pipe4Xi
from process4xi import Process4Xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from server4xi import Server4Xi
from state4xi import State4Xi
from token4xi import Token4Xi
from timeout4xi import omega2timeout4xi      
from print4xi import print4xi
from time4xi import time4xi
#"
#/>

# =================================================================================================
#<class4xi class = "BaseManager4Xi" base = "object" child = "SyncManager4Xi" synopsis = "Base class for managers" usage = "inherited by managers
#"
#>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class BaseManager4Xi(object):
    '''
    BaseManager4Xi: Base class for managers
    '''
     
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('get_server','connect','start','join','register',
                 'sense', 'reason', 'ace')         
    #"
    #/>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _registry = {}                     
    _Server = Server4Xi  
    _familyc2xi = ConstantFamily4Xi()
    #"
    #/>
        
                         
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-BaseManager4Xi" usage = "initializes">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, address=None, authkey=None, serializer='pickle'): 
        '''
        initializes BaseManager4Xi
        '''
                                
        # -----------------------------------------------------------------------------------------
        # (Step 1/8)
        # -----------------------------------------------------------------------------------------
    
        class2print = 'BaseManager4Xi'
        method2print = "__init__"
        
        message2print = "{(Step 1/8a) started" + "; init-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     

        message2print = "{(Step 1/8b) borning" + "; basemanager2xi-of-BaseManager4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{(Step 1/8c) inheriting" + "; object-inheriting-by-init-of-BaseManager4Xi to: " + "object"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                     

        message2print = "{(Step 1/8d) sharing" + "; registry-sharing-by-basemanager2xi-of-BaseManager4Xi: " + str(BaseManager4Xi._registry)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      

        message2print = "{(Step 1/8e) sharing" + "; Server-sharing-by-basemanager2xi-of-BaseManager4Xi: " + str(BaseManager4Xi._Server)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
                                    
        message2print = "{(Step 1/8f) passed" + "; address-passed-to-init-of-BaseManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 1/8g) passed" + "; authkey-passed-to-init-of-BaseManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 1/8h) passed" + "; serializer-passed-to-init-of-BaseManager4Xi: " + str(serializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
              
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/8) - DRAGONIZED AUTHORIZATION 
        # -----------------------------------------------------------------------------------------        
        if authkey is None:
            authkey = get_authkey_current_process4xi() 
            
            message2print = "{(Step 2/8) set; authkey-set-by-init-of-BaseManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)         
            # b'\x00\xd7R\xa2\x80\xea\xac:\xad\x91\xab\x1f\x1b\xcd\xa8O\x82-\x02:\xfb\xfb\x1cU\xc97\xbe)-;Lr'}
       
        # -----------------------------------------------------------------------------------------
        # (Step 3/8)
        # -----------------------------------------------------------------------------------------
        if address is None:
            message2print = "{(Step 3/8a) warned" + "; address-warned-by-init-of-BaseManager4Xi: " + " ** " + str(address)  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
                        
        
        self._address = address     # not final address if eg ('', 0)
        
        message2print = "{(Step 3/8b) set" + "; address-of-basemanager2xi-set-by-init-of-BaseManager4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
                
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/8)
        # -----------------------------------------------------------------------------------------                
        self._authkey = authkey
        
        message2print = "{(Step 4/8) set; authkey-of-basemanager2xi-set-by-init-of-BaseManager4Xi to : " + str(self._authkey) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/8)
        # -----------------------------------------------------------------------------------------                
        self._state = State4Xi()
        
        message2print = "{(Step 5/8) set; state-of-basemanager2xi-set-by-init-of-BaseManager4Xi to : " + str(self._state) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <state4xi.State4Xi object at 0x0000000002B578E8>}
        
        # -----------------------------------------------------------------------------------------
        # (Step 6/8)
        # -----------------------------------------------------------------------------------------                
        self._state.value = State4Xi.INITIAL
        
        message2print = "{(Step 6/8) set; state-value-of-basemanager2xi-set-by-init-of-BaseManager4Xi to : " + str(self._state.value) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)        
        # 0}
        
        # -----------------------------------------------------------------------------------------
        # (Step 7/8)
        # -----------------------------------------------------------------------------------------        
        self._serializer = serializer
        
        message2print = "{(Step 7/8) set; serializer-of-basemanager2xi-set-by-init-of-BaseManager4Xi to : " + str(self._serializer) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)        
        # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 8/8)
        # -----------------------------------------------------------------------------------------        
        self._Listener, self._Client = listener_client4xi[self._serializer]    
        
        message2print = "{(Step 8/8a) set; Listener-of-basemanager2xi-set-by-init-of-BaseManager4Xi to : " + str(self._Listener) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)         
        # <class 'listener4xi.Listener4Xi'>}
        
        message2print = "{(Step 8/8b) set; Client-of-basemanager2xi-set-by-init-of-BaseManager4Xi to : " + str(self._Client) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)
        # <function Client4Xi at 0x000000000379C8C8>}
        
        message2print = "{(Step 8/8c) inherited" + "; object-inherited-by-init-of-BaseManager4Xi to: " + "object"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        message2print = "{(Step 8/8d) activated" + "; basemanager2xi-of-BaseManager4Xi-activated: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 

        message2print = "{(Step 8/8e) shared" + "; registry-shared-by-basemanager2xi-of-BaseManager4Xi: " + str(BaseManager4Xi._registry)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      

        message2print = "{(Step 8/8f) shared" + "; Server-shared-by-basemanager2xi-of-BaseManager4Xi: " + str(BaseManager4Xi._Server)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
               
        message2print = "{(Step 8/8g) returning" + "; init-of-BaseManager4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
    #</methodize4xi>
            
               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reduce-of-BaseManager4Xi" usage = "reduces">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __reduce__(self):
        '''
        BaseManager4Xi method __reduce__: reduces 
        '''
        class2print = 'BaseManager4Xi'
        method2print = '__reduce__'
        
        message2print = "{started" + "; reduce-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        message2print = "{returning" + "; type-returning-by-reduce-of-BaseManager4Xi: " + str(type(self).from_address, (self._address, self._authkey, self._serializer)) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)     
        
        return type(self).from_address, \
               (self._address, self._authkey, self._serializer)   

        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-server-of-BaseManager4Xi" usage = "returns server object with serve_forever() method and address attribute">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def get_server(self):
        '''
        BaseManager4Xi method get_server: returns server object with serve_forever() method and address attribute
        '''
        class2print = 'BaseManager4Xi'
        method2print = 'get_server'
        
        
        message2print = "{started" + "; get-server-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        assert self._state.value == State4Xi.INITIAL
        
        message2print = "{asserted" + "; state-value-of-basemanager2xi-asserted-by-server-of-BaseManager4Xi: " + str(self._state.value) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                    
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{passing" + "; registry-of-basemanager2xi-passing-to-Server4Xi-by-server-of-BaseManager4Xi: " + str(self._registry) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                

        message2print = "{passing" + "; address-of-basemanager2xi-passing-to-Server4Xi-by-server-of-BaseManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{passing" + "; authkey-of-basemanager2xi-passing-to-Server4Xi-by-server-of-BaseManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{passing" + "; serializer-of-basemanager2xi-passing-to-Server4Xi-by-server-of-BaseManager4Xi: " + str(self._serializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                                    
                
        message2print = "{returning" + "; Server4Xi-class-returning-by-get-server-of-BaseManager4Xi: " + str(Server4Xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        
    
        return Server4Xi(self._registry, self._address,
                      self._authkey, self._serializer)


    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "connect-of-BaseManager4Xi" usage = "connects a manager object to the server process">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def connect(self):
        '''
        BaseManager4Xi method connect: connects a manager object to the server process
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------           
        class2print = 'BaseManager4Xi'
        method2print = 'connect'
        
        
        message2print = "{started" + "; connect-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------                
        message2print = "{passing" + "; serializer-of-basemanager2xi-passing-to-listener-client4xi-by-connect-of-BaseManager4Xi: " + str(self._serializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                
                  
        Listener2Xi, Client2Xi = listener_client4xi[self._serializer]
        
        message2print = "{set" + "; Listener2Xi-set-by-connect-of-BaseManager4Xi to: " + str(Listener2Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        
        message2print = "{set" + "; Client2Xi-set-by-connect-of-BaseManager4Xi to: " + str(Client2Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                   
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{passing" + "; address-of-basemanager2xi-passing-to-Client2Xi-by-connect-of-BaseManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        message2print = "{passing" + "; authkey-of-basemanager2xi-passing-to-Client2Xi-by-connect-of-BaseManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                           
        
       
        connection2xi = Client2Xi(self._address, authkey=self._authkey)
                
        message2print = "{set" + "; connection2xi-set-by-connect-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{passing" + "; connection2xi-passing-to-dispatch4xi-by-connect-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                
        message2print = "{passing" + "; dummy-passing-to-dispatch4xi-by-connect-of-BaseManager4Xi: " + str(get_dummy_manager4xi()) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        dispatch4xi(connection2xi, None, get_dummy_manager4xi()) #DUMMY_MANAGER4XI
        
        message2print = "{dispatched" + "; connection-dispatched-by-connect-of-BaseManager4Xi: "  + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        self._state.value = State4Xi.STARTED
        
        message2print = "{set" + "; basemanager2xi-state-value-set-by-connect-of-BaseManager4Xi to: " + str(self._state.value)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{returning" + "; connect-of-BaseManager4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "start-of-BaseManager4Xi" usage = "spawns a server process for this manager object">                            
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def start(self, initializer=None, initargs=()):
        '''
        BaseManager4Xi method start: spawns a server process for this manager object
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 0/11) 
        # -----------------------------------------------------------------------------------------
           
        class2print = 'BaseManager4Xi'
        method2print = 'start'
        
         
        message2print = "{(Step 0/11a) started" + "; start-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
        
        assert self._state.value == State4Xi.INITIAL
 
        message2print = "{(Step 0/11b) asserted" + "; basemanager2xi-state-value-asserted-by-start-of-BaseManager4Xi: " + "self._state.value" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        if initializer is not None and not callable(initializer):
            message2print = "{(Step 0/11z) failed" + "; initializer-failed-by-start-of-BaseManager4Xi: " + str(initializer) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            raise TypeError('basemanager4xi.py; BaseManager4Xi initializer must be a callable')
        
                       
        # -----------------------------------------------------------------------------------------
        # Step 1/11
        # -----------------------------------------------------------------------------------------
        # pipe over which we will retrieve address of server
        message2print = "{(Step 1/11a) passing" + "; False-passing-to-Pipe4Xi-by-start-of-BaseManager4Xi: " + str(False) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        
        reader2xi = None
        writer2xi = None               

        try:
            reader2xi, writer2xi = Pipe4Xi(duplex=False)
            
            message2print = "{(Step 1/11b) created; reader2xi-created-by-start-of-BaseManager4Xi: " + str(reader2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{(Step 1/11c) created; writer2xi-created-by-start-of-BaseManager4Xi: " + str(writer2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
        except:
            message2print = "{(Step 1/11z) failed; writer2xi-failed-by-start-of-BaseManager4Xi: " + str(writer2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
            
        # -----------------------------------------------------------------------------------------
        # Step 2/11
        # -----------------------------------------------------------------------------------------        
        # spawn process which runs a server
        
        message2print = "{(Step 2/11a) spawning" + "; process-spawning-by-start-of-BaseManager4Xi: " + str('running a server')  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        message2print = "{(Step 2/11b) passing" + "; run-server-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(type(self)._run_server) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                

        message2print = "{(Step 2/11c) passing" + "; registry-of-basemanager2xi-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(self._registry) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 2/11d) passing" + "; address-of-basemanager2xi-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                                  

        message2print = "{(Step 2/11e) passing" + "; authkey-of-basemanager2xi-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
             

        message2print = "{(Step 2/11f) passing" + "; serializer-of-basemanager2xi-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(self._serializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 2/11g) passing" + "; writer2xi-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(writer2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
          
        message2print = "{(Step 2/11h) passing" + "; initializer-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(initializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 2/11i) passing" + "; initargs-passing-to-Process4Xi-by-start-of-BaseManager4Xi: " + str(initargs) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        try: # usage as it has been before inearlier versions
            self._process = Process4Xi(target=type(self)._run_server,args=(self._registry, self._address, self._authkey, self._serializer, writer, initializer, initargs),)
            
            message2print = "{(Step 2/11j) spawned" + "; process-spawned-by-start-of-BaseManager4Xi: " + str(self._process) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
        except:
            message2print = "{(Step 2/11z) hit" + "; Process4Xi-hit-by-start-of-BaseManager4Xi: " + " *** failed to spawn process = " + str(Process4Xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            raise        
                                    
                        
        # -----------------------------------------------------------------------------------------
        # Step 3/11
        # -----------------------------------------------------------------------------------------            
        ident = None
        try:
            ident = ':'.join(str(i) for i in self._process._identity)
                    
            message2print = "{(Step 3/11) set; ident-set-by-start-of-BaseManager4Xi to: " + str(ident) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
        except:
            message2print = "{(Step 3/11z) failed; ident-failed-by-start-of-BaseManager4Xi: " + str(ident) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            
        # -----------------------------------------------------------------------------------------
        # Step 4/11
        # -----------------------------------------------------------------------------------------                
        self._process.name = None
        try:
            self._process.name = type(self).__name__  + '-' + ident
                    
            message2print = "{(Step 4/11 ) set; process-name-of-basemanager2xi-set-by-start-of-BaseManager4Xi to: " + str(self._process.name) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
        except:
            message2print = "{(Step 4/11z) failed; name-of-process-failed-by-start-of-BaseManager4Xi: " + str(self._process.name) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
            
        # -----------------------------------------------------------------------------------------
        # Step 5/11
        # -----------------------------------------------------------------------------------------                
        try:
            self._process.start()
            
            message2print = "{(Step 5/11a) performed" + "; process-start-of-basemanager2xi-performed-by-start-of-BaseManager4Xi: "  + str(self._process.start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        except:
            message2print = "{(Step 5/11z1) failed; start-of-process-failed-by-start-of-BaseManager4Xi: " + str(self._process.name) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
        
        try:    
            self._process.run()                                                    #dragonized - run added !!
            
            message2print = "{(Step 5/11b) performed" + "; process-run-of-basemanager2xi-performed-by-start-of-BaseManager4Xi: "  + str(self._process.run) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        except:
            message2print = "{(Step 5/11z2) failed; run-of-process-failed-by-start-of-BaseManager4Xi: " + str(self._process.name) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)        
                
        # -----------------------------------------------------------------------------------------
        # Step 6/11
        # -----------------------------------------------------------------------------------------        
        try:
            writer2xi.close()
            
            message2print = "{(Step 6/11 ) closed" + "; writer2xi-closed-by-start-of-BaseManager4Xi: " + str(writer2xi.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        except:
            message2print = "{(Step 6/11z) failed; close-of-process-failed-by-start-of-BaseManager4Xi: " + str(self._process.name) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        # Step 7/11
        # -----------------------------------------------------------------------------------------        
        # get address of server
        message2print = "{(Step 7/11a) receiving" + "; address-of-basemanager2xi-receiving-by-start-of-BaseManager4Xi: "  + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        self._address = None        
        try:
            self._address = reader2xi.recv()
        
            message2print = "{(Step 7/11a) received" + "; address-of-basemanager2xi-received-by-start-of-BaseManager4Xi: "  + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
        except:
            message2print = "{(Step 7/11b) hit" + "; reader2xi-recv-hit-by-start-of-BaseManager4Xi: " + " *** " + str(self._address)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
  
        
        # -----------------------------------------------------------------------------------------
        # Step 8/11
        # -----------------------------------------------------------------------------------------        
        try:
            reader2xi.close()
        
            message2print = "{(Step 8/11a) closed" + "; reader2xi-closed-by-start-of-BaseManager4Xi: " + str(reader2xi.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        except:
            message2print = "{(Step 8/11a) hit" + "; reader2xi-close-hit-by-start-of-BaseManager4Xi: " + " *** " + str(reader2xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)

        # -----------------------------------------------------------------------------------------
        # Step 9/11
        # -----------------------------------------------------------------------------------------        
        # register a finalizer
        self._state.value = None
        try:
            self._state.value = State4Xi.STARTED
            
            message2print = "{(Step 9/11 ) set; state-value-of-basemanager2xi-set-by-start-of-BaseManager4Xi to: " + str(self._state.value) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
        except:
            message2print = "{(Step 9/11z) failed" + "; value-failed-by-start-of-BaseManager4Xi: " + " *** " + str(self._state.value)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
                    
        # -----------------------------------------------------------------------------------------
        # Step 10/11
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 10/11a) calling; class-calling-by-start-of-BaseManager4Xi: Finalize4Xi; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)

        message2print = "{(Step 10/11b) passing" + "; basemanager2xi-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 10/11c) passing" + "; type-finalize-manager-of-basemanager2xi-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + str(type(self)._finalize_manager) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 10/11d) passing" + "; process-of-basemanager2xi-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + str(self._process) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 10/11e) passing" + "; address-of-basemanager2xi-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 10/11f) passing" + "; authkey-of-basemanager2xi-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 10/11g) passing" + "; state-of-basemanager2xi-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 10/11h) passing" + "; Client-of-basemanager2xi-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + str(self._Client) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 10/11i) passing" + ";  exitpriority-passing-to-Finalize4Xi-by-start-of-BaseManager4Xi: " + str( exitpriority) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                                     
                        
                
        try:
            self.shutdown = Finalize4Xi(self, type(self)._finalize_manager,
                                        args=(self._process, self._address, self._authkey, self._state, self._Client), exitpriority=0)
        
            
            message2print = "{(Step 10/11j) got" + "; basemanager2xi-shutdown-got-by-start-of-BaseManager4Xi: " + str(self.shutdown)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
        
        except:
            message2print = "{(Step 10/11k) hit" + "; Finalize4Xi-hit-by-start-of-BaseManager4Xi: " + " *** " + str(Finalize4Xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            raise               
            
        # -----------------------------------------------------------------------------------------
        # Step 11/11
        # ----------------------------------------------------------------------------------------- 
        
         
        
        message2print = "{(Step 11/11) returning" + "; start-of-BaseManager4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "run-server-of-BaseManager4Xi" usage = "creates a server, report its address and run it">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @classmethod
    def _run_server(cls, registry, address, authkey, serializer, writer,
                    initializer=None, initargs=()):
        '''
        BaseManager4Xi method _run_server: creates a server, report its address and run it
        '''
        # -----------------------------------------------------------------------------------------
        # Step 1/6
        # -----------------------------------------------------------------------------------------         
        class2print = 'BaseManager4Xi'
        method2print = '_run_server'
        
        ack2run4server = 0
        
        message2print = "{(Step 1/6a) started" + "; run-server-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
                
        message2print = "{(Step 1/6b) received" + "; cls-received-by-run-server-of-BaseManager4Xi: "  + str(cls) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        message2print = "{(Step 1/6c) received" + "; registry-by-run-server-of-BaseManager4Xi: "  + str(registry) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        message2print = "{(Step 1/6d) received" + "; address-by-run-server-of-BaseManager4Xi: "  + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        message2print = "{(Step 1/6e) received" + "; authkey-by-run-server-of-BaseManager4Xi: "  + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        message2print = "{(Step 1/6f) received" + "; serializer-by-run-server-of-BaseManager4Xi: "  + str(serializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{(Step 1/6g) received" + "; writer-by-run-server-of-BaseManager4Xi: "  + str(writer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 1/6h) received" + "; initializer-by-run-server-of-BaseManager4Xi: "  + str(initializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/6i) received" + "; initargs-by-run-server-of-BaseManager4Xi: "  + str(initargs) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        
        ack2run4server = ack2run4server + 1
                
        # -----------------------------------------------------------------------------------------
        # Step 2/6
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 2/6a) passing" + "; registry-passing-to-cls-Server-by-run-server-of-BaseManager4Xi: " + str(registry) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        message2print = "{(Step 2/6b) passing" + "; address-passing-to-cls-Server-by-run-server-of-BaseManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                          
        
        message2print = "{(Step 2/6c) passing" + "; authkey-passing-to-cls-Server-by-run-server-of-BaseManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                          
        
        message2print = "{(Step 2/6d) passing" + "; serializer-passing-to-cls-Server-by-run-server-of-BaseManager4Xi: " + str(serializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        message2print = "{(Step 2/6e) calling" + "; cls-Server-calling-by-server-of-BaseManager4Xi: " + str(cls._Server) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                
        if initializer is not None:
            initializer(*initargs)
            
        server2xi = None
        try:                
            server2xi = cls._Server(registry, address, authkey, serializer)  # refers to parameter cls
            
            message2print = "{(Step 2/6f) created; server-object-created-by-run-server-of-BaseManager4Xi: " + str(server2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
            ack2run4server = ack2run4server + 1

        except:
            message2print = "{(Step 2/6z) hit" + "; server2xi-hit-by-run-server-of-BaseManager4Xi: " + " *** " + str(server2xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                        
                          
            
        # -----------------------------------------------------------------------------------------
        # Step 3/6
        # -----------------------------------------------------------------------------------------        
        # inform parent process of the server's address
        
        message2print = "{(Step 3/6a) passing" + "; server2xi-address-passing-to-writer-send-by-run-server-of-BaseManager4Xi: " + str(server2xi.address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        try:                
        
            writer.send(server2xi.address)
        
            message2print = "{(Step 3/6b) sent; server-address-sent-by-run-server-of-BaseManager4Xi:  " + str(server2xi.address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            ack2run4server = ack2run4server + 1 
        except:
            message2print = "{(Step 3/6z) hit" + "; writer-send-hit-by-run-server-of-BaseManager4Xi: " + " *** " + str(writer.send)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                     
            
        # -----------------------------------------------------------------------------------------
        # Step 4/6 
        # -----------------------------------------------------------------------------------------                
        
        try:
            writer.close()
        
            message2print = "{(Step 4/6a) closed" + "; writer-closed-by-run-server-of-BaseManager4Xi: " + str(writer.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            ack2run4server = ack2run4server + 1           
        except:
            message2print = "{(Step 4/6z) hit" + "; writer-close-hit-by-run-server-of-BaseManager4Xi: " + " *** " + str(writer.close)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
        # -----------------------------------------------------------------------------------------
        # Step 5/6 
        # -----------------------------------------------------------------------------------------        
        
        if ack2run4server == 4:
        
            #manager serving
        
            message2print = "{(Step 5/6a) serving" + "; server2xi-address-serving-by-run-server-of-BaseManager4Xi: " + str(server2xi.address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
        
            message2print = "{(Step 5/6b) running" + "; manager-address-running-by-run-server-of-BaseManager4Xi: " + str(server2xi.address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            server2xi.serve_forever()
             
        else:
     
            message2print = "{(Step 5/6c) hit" + "; ack2run4server-hit-by-run-server-of-BaseManager4Xi: " + " *** " + str(ack2run4server)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                                           
        # -----------------------------------------------------------------------------------------
        # Step 6/6 
        # -----------------------------------------------------------------------------------------        
     
        message2print = "{(Step 6/6) returning" + "; serve-forever-of-Server-returning-by-run-server-of-BaseManager4Xi: " + str(server2xi.serve_forever) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)          
        
        
        
    #</methodize4xi>
    
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create-of-BaseManager4Xi" usage = "creates a new shared object; return the token and exposed tuple">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _create(self, typeid, *args, **kwds):
        '''
        BaseManager4Xi method _create: creates a new shared object; return the token and exposed tuple
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)
        # ----------------------------------------------------------------------------------------- 
        class2print = 'BaseManager4Xi'
        method2print = '_create'
        
        message2print = "{(Step 1/5a) started" + "; create-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        message2print = "{(Step 1/5b) passed" + "; typeid-passed-to-create-of-BaseManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        message2print = "{(Step 1/5c) passed" + "; args-passed-to-create-of-BaseManager4Xi: " + str(*args) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        message2print = "{(Step 1/5d) passed" + "; kwds-passed-to-create-of-BaseManager4Xi: " + str(**kwds) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      

                        
        # -----------------------------------------------------------------------------------------
        # (Step 2/5)
        # ----------------------------------------------------------------------------------------- 
               
        assert self._state.value == State4Xi.STARTED, 'BaseManager4Xi:_create: server not yet started'
        
        message2print = "{(Step 2/5a) asserted" + "; state-of-basemanager2xi-asserted-by-create-of-BaseManager4Xi: " + str(self._state.value) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 2/5b) passing" + "; address-of-basemanager2xi-passing-to-Client-of-basemanager2xi-by-create-of-BaseManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        
        message2print = "{(Step 2/5c) passing" + "; authkey-of-basemanager2xi-passing-to-Client-of-basemanager2xi-by-create-of-BaseManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                             
        
       
        connection2xi = self._Client(self._address, BaseManager4Xi._familyc2xi.get_default_family(), self._authkey)
        
        message2print = "{(Step 2/5d) set" + "; connection2xi-set-by-create-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
   
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 3/5)
            # -----------------------------------------------------------------------------------------     
            message2print = "{(Step 3/5a) passing" + "; connection2xi-passing-to-dispatch4xi-by-create-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
 
            message2print = "{(Step 3/5b) dispatching" + "; connection-dispatching-by-create-of-BaseManager4Xi: "  + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            id2xi, exposed2xi = dispatch4xi(connection2xi, None, get_create_manager4xi(), (typeid,)+args, kwds) #CREATE_MANAGER4XI
            
            message2print = "{(Step 3/5c) dispatched" + "; id-connection-dispatched-by-create-of-BaseManager4Xi: "  + str(id2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            message2print = "{(Step 3/5d) dispatched" + "; exposed2xi-dispatched-by-create-of-BaseManager4Xi: "  + str(exposed2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2create4xi = ack2create4xi + 1  
        finally:
            # -----------------------------------------------------------------------------------------
            # (Step 4/5)
            # -----------------------------------------------------------------------------------------          
            message2print = "{(Step 4/5a) closing" + "; connection-closing-by-create-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            
            connection2xi.close()
            
            message2print = "{(Step 4/5b) closed" + "; connection-closed-by-create-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                   
        # -----------------------------------------------------------------------------------------
        # (Step 5/5)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 5/5a) passing" + "; typeid-passing-to-Token4Xi-by-create-of-BaseManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        message2print = "{(Step 5/5b) passing" + "; address-of-basemanager2xi-passing-to-Token4Xi-by-create-of-BaseManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 5/5c) passing" + "; id-passing-to-Token4Xi-by-create-of-BaseManager4Xi: " + str(id) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                  
        
        message2print = "{(Step 5/5d) returning" + "; Token4Xi-class-returning-by-create-of-BaseManager4Xi: " + str(Token4Xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 5/5e) returning" + "; exposed-returning-by-create-of-BaseManager4Xi: " + str(exposed) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                              
            
        return Token4Xi(typeid, self._address, id), exposed

    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "join-of-BaseManager4Xi" usage = "joins the manager process (if it has been spawned)">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def join(self, timeout=None):
        '''
        BaseManager4Xi method join: joins the manager process (if it has been spawned)
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------
        class2print = 'BaseManager4Xi'
        method2print = 'join'
        
        
        message2print = "{(Step 1/3a) started" + "; join-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  

        message2print = "{(Step 1/3b) passed" + "; timeout-passed-to-join-of-BaseManager4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        timeout = omega2timeout4xi(class2print, method2print)         
        
        message2print = "{(Step 1/3c) set" + "; timeout-set-by-join-of-BaseManager4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
                        
                
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/3a) joining" + "; process-of-basemanager2xi-joining-by-join-of-BaseManager4Xi: "  + str(self._process.join) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        
        if self._process is not None:
            self._process.join(timeout)
            if not self._process.is_alive():
                self._process = None
                message2print = "{(Step 2/3z1) failed" + "; process-of-basemanager2xi-failed-by-join-of-BaseManager4Xi: "  + str(self._process) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
            else:
                message2print = "{(Step 2/3b) joined" + "; process-of-basemanager2xi-joined-by-join-of-BaseManager4Xi: "  + str(self._process.join) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 2/3z2) failed" + "; process-of-basemanager2xi-failed-by-join-of-BaseManager4Xi: "  + str(self._process) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3/3) returning" + "; join-of-BaseManager4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)     
       
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "debug-info-of-BaseManager4Xi" usage = "returns some info about the servers shared objects and connections">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _debug_info(self):
        '''
        BaseManager4Xi method _debug_info: returns some info about the servers shared objects and connections
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------          
        class2print = 'BaseManager4Xi'
        method2print = '_debug_info'
        
        message2print = "{started" + "; debug-info-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                                
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        connection2xi = self._Client(self._address, authkey=self._authkey)
        
        message2print = "{created; connection-object-created-by-debug-info-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        try:
            message2print = "{returning; function-returning-by-debug-info-of-BaseManager4Xi: dispatch4xi; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            return dispatch4xi(connection2xi, None, get_debug_info_manager4xi()) #DEBUG_INFO_MANAGER4XI
        
        finally:
            message2print = "{closing; connection-closing-by-debug-info-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            connection2xi.close()
        
            message2print = "{returning" + "; debug-info-of-BaseManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
        
            
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "number-of-objects-of-BaseManager4Xi" usage = "returns the number of shared objects">                            
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _number_of_objects(self):
        '''
        BaseManager4Xi method _number_of_objects: returns the number of shared objects
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------          
        class2print = 'BaseManager4Xi'
        method2print = '_number_of_objects'
        
        
        message2print = "{started" + "; number-of-objects-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{passing" + "; address-of-basemanager2xi-passing-to-Client-by-number-of-objects-of-BaseManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                
        message2print = "{calling" + "; Client-of-basemanager2xi-calling-by-number-of-objects-of-BaseManager4Xi: " + str(self._Client) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                        

        connection2xi = self._Client(self._address, authkey=self._authkey)
        
        message2print = "{created; connection-created-by-number-of-objects-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        try:
            message2print = "{passing" + "; connection2xi-passing-to-dispatch4xi-by-number-of-objects-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            message2print = "{trying; return-trying-by-number-of-objects-of-BaseManager4Xi: dispatch4xi; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            return dispatch4xi(connection2xi, None, get_number_of_objects_manager4xi()) #NUMBER_OF_OBJECTS_MANAGER4XI
            
        finally:
            message2print = "{closing; connection-closing-by-number-of-objects-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
            
            connection2xi.close()
        
            message2print = "{returning" + "; number-of-objects-of-BaseManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
            
            
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "enter-of-BaseManager4Xi" usage = "returns self">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __enter__(self):
        '''
        BaseManager4Xi method __enter__: 
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        class2print = 'BaseManager4Xi'
        method2print = '__enter__'
        
        
        message2print = "{started" + "; enter-of-BaseManager4Xi-started: " + str(__enter__)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        if self._state.value == State4Xi.INITIAL:
            self.start()
        assert self._state.value == State4Xi.STARTED  
        
        
        message2print = "{returning" + "; enter-of-basemanager2xi-returning-by-enter-of-BaseManager4Xi: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)          
        
        return self   
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "exit-of-BaseManager4Xi" usage = "exits">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __exit__(self, exc_type, exc_val, exc_tb):
        '''
        BaseManager4Xi method __exit__:
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        class2print = 'BaseManager4Xi'
        method2print = '__exit__'
        
        
        
        message2print = "{started" + "; exit-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
          
        try:
        
            self.shutdown()   

            message2print = "{performed" + "; shutdown-of-basemanager2xi-performed-by-exit-of-BaseManager4Xi: " + str(self.shutdown) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
        except:
            message2print = "{hit" + "; shutdown-of-basemanager2xi-hit-by-exit-of-BaseManager4Xi: " + " *** " + str(self.shutdown)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
        message2print = "{returning" + "; exit-of-BaseManager4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)               
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
    #</methodize4xi>
            
        
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "finalize-manager-of-BaseManager4Xi" usage = "
    #         start(self) <--- funtion in this module
    #         self.shutdown = util.Finalize( <--- makes sure util module is used
    #             self, type(self)._finalize_manager, <--- here 
    #             args=(self._process, self._address, self._authkey,
    #                   self._state, self._Client),
    #             exitpriority=0
    #             )">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @staticmethod
    def _finalize_manager(process, address, authkey, state, _Client):
        '''
        BaseManager4Xi method _finalize_manager: shuts down the manager process; will be registered as a finalizer
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------
             
        class2print = 'BaseManager4Xi'
        method2print = '_finalize_manager'
        
        
        message2print = "{(Step 1/6a) started" + "; finalize-manager-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
        message2print = "{(Step 1/6b) passed" + "; process-passed-to-finalize-manager-of-BaseManager4Xi: " + str(process) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        message2print = "{(Step 1/6c) passed" + "; address-passed-to-finalize-manager-of-BaseManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        message2print = "{(Step 1/6d) passed" + "; authkey-passed-to-finalize-manager-of-BaseManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        message2print = "{(Step 1/6e) passed" + "; state-passed-to-finalize-manager-of-BaseManager4Xi: " + str(state) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        message2print = "{(Step 1/6f) passed" + "; Client-passed-to-finalize-manager-of-BaseManager4Xi: " + str(_Client) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        if process.is_alive():
            # -----------------------------------------------------------------------------------------
            # (Step 2/6)
            # -----------------------------------------------------------------------------------------
           
            # sending shutdown message to manager
            
            message2print = "{(Step 2/6a) sending; shutdown-message-sending-by-finalize-manager-of-BaseManager4Xi: " + str(_Client)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
            
            connection2xi = None
            try:
                message2print = "{(Step 2/6b) passing" + "; address-passing-to-Client-by-finalize-manager-of-BaseManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 2/6c) passing" + "; authkey-passing-to-Client-by-finalize-manager-of-BaseManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                             
                
                connection2xi = _Client(address, authkey=authkey)
                
                message2print = "{(Step 2/6e) created; connection-object-created-by-finalize-manager-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
            except:
                message2print = "{(Step 2/6z1) failed; connection-object-failed-by-finalize-manager-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                    
            try:
                message2print = "{(Step 2/6f) calling; function-calling-by-finalize-manager-of-BaseManager4Xi: dispatch4xi; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                    
                dispatch4xi(connection2xi, None, 'shutdown')
                    
                message2print = "{(Step 2/6g) performed; function-performed-by-finalize-manager-of-BaseManager4Xi: dispatch4xi; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            finally:
                try:
                    connection2xi.close()
                    
                    message2print = "{(Step 2/6h) closed; connection-object-closed-by-finalize-manager-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                except:
                    pass      

            # -----------------------------------------------------------------------------------------
            # (Step 3/6)
            # -----------------------------------------------------------------------------------------
            timeout = omega2timeout4xi(class2print, method2print)
            
            message2print = "{(Step 3/6a) set" + "; timeout-set-by-finalize-manager-of-BaseManager4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                            
            message2print = "{(Step 3/6b) joining; timeout-joining-by-finalize-manager-of-BaseManager4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
            
            try:
                process.join(timeout)
                message2print = "{(Step 3/6c) joined; process-joined-by-finalize-manager-of-BaseManager4Xi: " + str(process) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            except:
                message2print = "{(Step 3/6z1) failed; join-of-process-failed-by-finalize-manager-of-BaseManager4Xi: " + str(process) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   

            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 4/6a) checking; alive-status-checking-by-finalize-manager-of-BaseManager4Xi: " + str(process) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
            if process.is_alive():
                # -----------------------------------------------------------------------------------------
                # (Step 4/6) - MANAGER STILL ALIVE
                # -----------------------------------------------------------------------------------------
                message2print = "{(Step 4/6b) discovered" + "; manager-status-discovered-by-finalize-manager-of-BaseManager4Xi: still alive" + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                   
                if hasattr(process, 'terminate'):
                    # trying to terminate manager process
                    
                    message2print = "{(Step 4/6c1) terminating" + "; process-terminating-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
                    
                    try:
                        process.terminate()
                        message2print = "{(Step 4/6c2) terminated" + "; process-terminated-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                    except:
                        message2print = "{(Step 4/6z2) failed" + "; termination-of-process-failed-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)     
                    
                    message2print = "{(Step 4/6d1) joining" + "; process-joining-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
                    
                    timeout = omega2timeout4xi(class2print, method2print)
                    
                    message2print = "{(Step 4/6d2) set" + "; timeout-set-by-finalize-manager-of-BaseManager4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                    try:
                        process.join(timeout)
                    
                        message2print = "{(Step 4/6d3) joined" + "; process-joined-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                    except:
                        message2print = "{(Step 4/6z3) failed" + "; join-of-process-failed-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)     
                    
                        
                    message2print = "{(Step 4/6e1) checking; alive-status-checking-by-finalize-manager-of-BaseManager4Xi: " + str(process) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    if process.is_alive():
                        # manager still alive after terminate
                        message2print = "{(Step 4/e2) discovered" + "; alive-status-discovered-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                    else:
                        message2print = "{(Step 4/6e3) discovered" + "; dead-status-discovered-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                # -----------------------------------------------------------------------------------------
                #
                # -----------------------------------------------------------------------------------------
            else:
                message2print = "{(Step 4/6f) discovered" + "; dead-status-discovered-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/6g) discovered" + "; dead-status-discovered-by-finalize-manager-of-BaseManager4Xi: " + str(process)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  

        # -----------------------------------------------------------------------------------------
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------
        try:
            state.value = State4Xi.SHUTDOWN
            
            message2print = "{(Step 5/6a) set; state-value-set-by-finalize-manager-of-BaseManager4Xi to: " + str(state.value) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
        except:
            message2print = "{(Step 5/6z) failed; value-of-state-failed-by-finalize-manager-of-BaseManager4Xi to: " + str(state) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 

        message2print = "{(Step 5/6b) deleting; proxy-object-deleting-by-finalize-manager-of-BaseManager4Xi: " + str(BaseProxy4Xi._address_to_local[address]) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                 
        try:
            del BaseProxy4Xi._address_to_local[address]
        except KeyError:
            message2print = "{(Step 5/6c) hit" + "; proxy-key-error-hit-by-finalize-manager-of-BaseManager4Xi: "  + str(address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
         
        # -----------------------------------------------------------------------------------------
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6/6) returning" + "; finalize-manager-of-BaseManager4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                
           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    # class  variables
    # ---------------------------------------------------------------------------------------------  
    #<define4xi definition = "
    address = property(lambda self: self._address)
    #"
    #/>        
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "register-of-BaseManager4Xi" usage = "registers a typeid with the manager type and return proxy">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @classmethod
    def register(cls, typeid, callable=None, proxytype=None, exposed=None, method_to_typeid=None, create_method=True):
        '''
        BaseManager4Xi method register: registers a typeid with the manager type and return proxy
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/10)
        # -----------------------------------------------------------------------------------------          
        class2print = 'BaseManager4Xi'
        method2print = 'register'
    
        message2print = "{(Step 1/10a) started" + "; register-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        

        message2print = "{(Step 1/10b) passed" + "; cls-passed-to-register-of-BaseManager4Xi: " + str(cls) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{(Step 1/10c) passed" + "; typeid-passed-to-register-of-BaseManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{(Step 1/10d) passed" + "; callable-passed-to-register-of-BaseManager4Xi: " + str(callable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{(Step 1/10e) passed" + "; proxytype-passed-to-register-of-BaseManager4Xi: " + str(proxytype) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{(Step 1/10f) passed" + "; exposed-passed-to-register-of-BaseManager4Xi: " + str(exposed) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{(Step 1/10g) passed" + "; method-to-typeid-passed-to-register-of-BaseManager4Xi: " + str(method_to_typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
                               
        message2print = "{(Step 1/10h) passed" + "; create-method-passed-to-register-of-BaseManager4Xi: " + str(create_method) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/10)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 2/10a) checking" + "; cls-dict-checking-by-register-of-BaseManager4Xi: " + str(cls.__dict__) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
                        
        
        if '_registry' not in cls.__dict__:
        
            message2print = "{(Step 2/10b) calling" + "; cls-registry-copy-calling-by-register-of-BaseManager4Xi: " + str(cls._registry.copy) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                            
                                    
            cls._registry = cls._registry.copy()
            
            message2print = "{(Step 2/10c) set; cls-registry-set-by-register-of-BaseManager4Xi to: " + str(cls._registry) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)

        # -----------------------------------------------------------------------------------------
        # (Step 3/10)
        # -----------------------------------------------------------------------------------------            
        if proxytype is None:
            proxytype = AutoProxy4Xi
            
            message2print = "{(Step 3/10) created; proxytype-created-by-register-of-BaseManager4Xi: " + str(proxytype) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/10)
        # -----------------------------------------------------------------------------------------        
        exposed = exposed or getattr(proxytype, '_exposed_', None)
        
        message2print = "{(Step 4/10) set; exposed-set-by-register-of-BaseManager4Xi to: " + str(exposed) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/10)
        # -----------------------------------------------------------------------------------------        
        method_to_typeid = method_to_typeid or \
                           getattr(proxytype, '_method_to_typeid_', None)
        
        message2print = "{ (Step 5/10) set; method-to-typeid-set-by-register-of-BaseManager4Xi to: " + str(method_to_typeid) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # (Step 6/10)
        # -----------------------------------------------------------------------------------------                           
        if method_to_typeid:
            for key, value in list(method_to_typeid.items()):
                assert type(key) is str, '%r is not a string' % key
                assert type(value) is str, '%r is not a string' % value
                
                message2print = "{(Step 6/10a) asserted" + "; key-asserted-by-register-of-BaseManager4Xi: " + str(key) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                      
                
                message2print = "{(Step 6/10b) asserted" + "; value-asserted-by-register-of-BaseManager4Xi: " + str(value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                      
              
        message2print = "{(Step 6/1c) asserted" + "; method-to-typeid-asserted-by-register-of-BaseManager4Xi: " + "method_to_typeid" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # (Step 7/10)
        # -----------------------------------------------------------------------------------------                
        message2print = "{(Step 7/10a) using" + "; typeid-using-by-register-of-BaseManager4Xi: " + str(typeid)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
        
        cls._registry[typeid] = (
            callable, exposed, method_to_typeid, proxytype
            )
        
        message2print = "{(Step 7/10b) set; cls-registry-set-by-register-of-BaseManager4Xi to: " + str(cls._registry[typeid]) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # (Step 8/10)
        # -----------------------------------------------------------------------------------------            
        if create_method:
            message2print = "{(Step 8/10) decided" + "; create_method-decided-by-register-of-BaseManager4Xi: "  + str(create_method) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            # -----------------------------------------------------------------------------------------
            #<function4xi function = "temp" usage = "
            #   requesting creation of a shared object
            #"> 
            #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
            # -----------------------------------------------------------------------------------------              
            def temp(self, *args, **kwds):
            
                # -------------------------------------------------------------------------------------
                #
                # -------------------------------------------------------------------------------------                       
                message2print = "{started" + "; temp-of-BaseManager4Xi-started: " + str(temp)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                        
                message2print = "{passed" + "; basemanager2xi-passed-to-temp-of-BaseManager4Xi: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                        
         
                # -------------------------------------------------------------------------------------
                #
                # -------------------------------------------------------------------------------------                       
                message2print = "{passing" + "; typeid-passing-to-create-of-basemanager2xi-by-temp-of-BaseManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
                

                
                token, exp = self._create(typeid, *args, **kwds)
                
                message2print = "{set" + "; token-set-by-temp-of-BaseManager4Xi to: " + str(token)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                        
                
                message2print = "{set" + "; exp-set-by-temp-of-BaseManager4Xi to: " + str(exp)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                   
                            
                # -------------------------------------------------------------------------------------
                #
                # -------------------------------------------------------------------------------------                       
    
                proxy = proxytype(
                    token, self._serializer, manager=self,
                    authkey=self._authkey, exposed=exp
                    )
                
                message2print = "{set" + "; proxy-set-by-temp-of-BaseManager4Xi to: " + str(proxy)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                                 
                # -------------------------------------------------------------------------------------
                #
                # -------------------------------------------------------------------------------------                       
    
                message2print = "{passing" + "; token-address-passing-to-Client-by-temp-of-BaseManager4Xi: " + str(token.address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                
                message2print = "{passing" + "; authkey-passing-to-Client-by-temp-of-BaseManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                
                connection2xi = self._Client(self._address, BaseManager4Xi._familyc2xi.get_default_family(), self._authkey) 
                
                message2print = "{connected" + "; connection2xi-connected-by-temp-of-BaseManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                
                dispatch4xi(connection2xi, None, 'decref', (token.id,))
                
                message2print = "{dispatched" + "; connection2xi-dispatched-by-temp-of-BaseManager4Xi: "  + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                message2print = "{returning" + "; proxy-returning-by-temp-of-BaseManager4Xi: " + str(proxy) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)                   
                
                return proxy
            
                # -------------------------------------------------------------------------------------
                #
                # -------------------------------------------------------------------------------------                       
            #</function4xi>
            
            # -----------------------------------------------------------------------------------------
            # (Step 9/10)
            # -----------------------------------------------------------------------------------------                
            temp.__name__ = typeid
            
            message2print = "{(Step 9/10a) set" + "; temp-name-set-by-register-of-BaseManager4Xi to: " + str(temp.__name__)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                

            message2print = "{(Step 9/10b) passing" + "; cls-passing-to-setattr-by-register-of-BaseManager4Xi: " + str(cls) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{(Step 9/10c) passing" + "; typeid-passing-to-setattr-by-register-of-BaseManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            message2print = "{(Step 9/10d) passing" + "; temp-passing-to-setattr-by-register-of-BaseManager4Xi: " + str(temp) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                                        
            setattr(cls, typeid, temp)
            
            message2print = "{(Step 9/10e) performed" + "; setattr-performed-by-register-of-BaseManager4Xi: " + str(setattr) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
        # -----------------------------------------------------------------------------------------
        # (Step 10/10)
        # -----------------------------------------------------------------------------------------              
        message2print = "{(Step 10/10) returning" + "; register-of-BaseManager4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
    #</methodize4xi>
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-BaseManager4Xi" usage = "senses BaseManager4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses BaseManager4Xi class objects
        '''
   
        
        class2print = 'BaseManager4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{sensing" + "; BaseManager4Xi-sensing-by-sense-of-BaseManager4Xi: " + str(BaseManager4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        kb2xi = sense_object4xi(self)

        try:
            kb2xi['inheritance-of-basemanager2xi'] = "object-inherited-by-init-of-BaseManager4Xi"
            kb2xi['registry-of-basemanager2xi'] = str(BaseManager4Xi._registry)           
            kb2xi['Server-of-basemanager2xi'] = str(BaseManager4Xi._Server)
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-BaseManager4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
 
        
        message2print = "{sensed" + "; BaseManager4Xi-sensed-by-sense-of-BaseManager4Xi: " + str(BaseManager4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-BaseManager4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)     
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-BaseManager4Xi" usage = "reasons BaseManager4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons BaseManager4Xi class objects
        '''
      
        
        class2print = 'BaseManager4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
  
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; BaseManager4Xi-reasoning-by-reason-of-BaseManager4Xi: "  + str(BaseManager4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        
        kb4xi = reason_object4xi(self)
        
        
        
        message2print = "{reasoned" + "; BaseManager4Xi-reasoned-by-reason-of-BaseManager4Xi: "  + str(BaseManager4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-BaseManager4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)              
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-BaseManager4Xi" usage = "aces BaseManager4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces BaseManager4Xi class objects 
        '''
     
        
        
        class2print = 'BaseManager4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-BaseManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
             
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; BaseManager4Xi-acing-by-ace-of-BaseManager4Xi: " + "BaseManager4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
      
        kb8xi = ace_object4xi(self)  
        
        message2print = "{aced" + "; BaseManager4Xi-aced-by-ace-of-BaseManager4X: " + "BaseManager4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-BaseManager4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
    #</methodize4xi>
                           
#</class4xi>
#</program4xi> 


# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#
#<record4xi record = "
#
#   test_proxy_mp_xi4.py.txt
#   sense-proxy-mp-test4xi.xml
#   reason-proxy-mp-test4xi.xml
#   ace-proxy-mp-test4xi.xml
#"
#>
  


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of basemanager4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>



